#!/usr/bin/octave
#
# $Id: ha-03.m,v 1.2 2010/01/22 20:32:51 demon Exp demon $
#
# Grundlagen der Raumfahrttechnik WS 2009/10
#	Hausaufgabe 3
# 
# Gruppe 14
#	Stefan Breit (310692)
#	Dimitri Sokolyuk (321116)
#

clear
# Prevent Octave from thinking that this is a function file:
1;

########################################################################
#
# Konstanten
#
g = 9.81;				# Schwerebeschleunigung [m/s^2]
RR = 8.31451;				# Universelle Gaskonstante [J/kg*K]

rho_TS = 1772.0;			# Mittlere TS-Dichte [kg/m^3]
M_TS = 27.8;				# Mittleres Molekulargewicht [kg/(mol*K)]
n_TS = 0.33;				# TS-Druckexponent [-]
a_TS = 0.0433;				# TS-Temperaturfaktor a [-]

kappa = 1.18;				# Verhaeltnis der spez. Waermen. [-]

p_c = 6400000;				# Druck (Brennbeginn) [N/m^2]
T_c = 3480;				# Temperatur [K]

I_sp = 241.5;				# Spez. Impuls (sea level) [s]
c_e = I_sp * g;
F = 4272.528 * 10^3;			# Schub (sea level) [N <- kN]

sigma_housing = 170000 * 6894.75;	# Zugfestigkeit [kg/m^2 <- lb/in^2]
					# 1 lb/in^2 = 6894.75 Pa (N/m^2)
rho_housing = 1992.95;			# Dichte [kg/m^3]

d_iso = 0.04;				# Dicke (zw. Wand und TS) [m]
rho_iso = 1090.59;			# Dichte [kg/m^3]

D_b = 3.1;				# Boosterduchmesser [m]
p_e = 66000;				# Duesenenddruck [N/m^2]

rueckstand = 0.02;			# Unverbrante TS-Rueckstaende [- <- %]
Sf = 2.5;				# Sicherheitsfaktor fuer Wanddickeberechunug
t_b = 131;				# Brennzeit [s]

########################################################################
#
# Aufgabe 3.1
#

# Teibstoffmassenstrom [kg/s]
m_dot = F / c_e

# Treibstoffmasse incl. Rueckstand [kg]
m = m_dot * t_b / (1 - rueckstand)

# Treibstoffvolumen
V = m / rho_TS

# Abbrandrate [m/s]
r_dot = a_TS * p_c^n_TS / 1000		# [mm/s -> m/s]

# Nottwendige Abbrandoberflaeche
A_not = m_dot / (r_dot * rho_TS)

# Webb-Thickness
b = r_dot * t_b

# Aussendruck
p_a = 0;

# Wanddicke
d_w = (p_c - p_a) * (D_b / 2) * Sf / (2 * sigma_housing)

# Treibsatz Aussenduchmesser
D_a = D_b - 2 * (d_w + d_iso)

# Treibsatz Innenduchmesser
D_i = D_a - 2 * b

function rad = d2r(degree)
# Degree -> Rad
	rad = degree * pi / 180;
endfunction

function degree = r2d(rad)
# Rad -> Degree
	degree = rad * 180 / pi;
endfunction

function [A V l] = endstuck(ra, ri)
# TS Volumen, Oberflaeche und TS Laenge an beiden Endstuecken
# [A V l] = endstuck(ra, ri)
	a = sqrt(ra^2 - ri^2);
	l = 2 * a;
	h = ra - a;
	V_kallote = 1/3 * h^2 * pi * (3 * ra - h);
	V_zylinder = pi * ri^2 * l;
	V_kugel = 4/3 * pi * ra^3;
	V = V_kugel - V_zylinder - 2 * V_kallote;
	A = 2 * pi * ri * l;
endfunction

function [A_tat l_zyl l_end] = tatsaechlich(V, ra, ri) # A()
# TS Gesamtlaenge und Abbrandoberflaeche
# [A_tat l_zyl l_end] = tatsaechlich(V, d_i, d_a)
	[A_end V_end l_end] = endstuck(ra, ri);

	# Restvolumen des zylindrischen Teils des TS
	V_zyl = V - V_end;

	# Laenge und Oberflaeche des zylindrischen Teils
	l_zyl = V_zyl / (pi * (ra^2 - ri^2));
	A_zyl = 2 * pi * ri * l_zyl;

	# Gesamtabbrandoberflaeche
	A_tat = A_zyl + A_end;
endfunction

function [A V] = spalt(ra, ri, alpha)
# [A V] = spalt(ra, ri, alpha)
	h = (ra - ri) * tan(alpha);
	s = (ra - ri) * sec(alpha);

	A = 2 * (ra + ri) * pi * s;	# Mantelflaeche
	V = 2 * (pi * ra^2 * h - 1/3 * h * pi * (ra^2 + ra * ri + ri^2));
endfunction

#####
function [dA dl] = zusatzoberflaeche(ra, ri, n, alpha)
# [dA dl] = zusatzoberflaeche(ra, ri, n, alpha)
# TS Zusatzoverflaeche und Zusatzlaenge
	[A_spalt V_spalt] = spalt(ra, ri, alpha);

	# Zusatzlaenge und Zusatzoberflaeche
	l_zus = V_spalt / (pi * (ra^2 - ri^2));
	A_zus = 2 * pi * ri * l_zus;

	dA = n * (A_spalt + A_zus);
	dl = n * l_zus;			# onhe 3cm Spalt
endfunction

function A = regression(t, r_dot, ra, ri, n, alpha, l_zyl)
	# Regression der Dicke
	db = r_dot * t;
	dr = ri + db;

	# Regression der Laenge an den Endstuecken
	[A_end V_end l_end] = endstuck(ra, dr);

	# Regression der Laenge an den Schlitzen
	dl = 2 * n * db * (sec(alpha) + tan(alpha));

	# Oberflaeche der Spalte
	[A_spalt V_spalt] = spalt(ra, dr, alpha);

	# Oberflaeche des zylindrischen Teils
	A_zyl = 2 * pi * dr * (l_zyl - dl);

	# Gesammtflaeche
	A = A_zyl + A_end + n * A_spalt;
endfunction

[A_tat l_zyl l_end] = tatsaechlich(V, D_a/2, D_i/2);
delta_A = A_not - A_tat;

# solution list initialization
spaltgeom = zeros(1, 6);
for n = 1:1:5
	for alpha = 0:0.5:60
		[dA dl] = zusatzoberflaeche(D_a/2, D_i/2, n, d2r(alpha));
		# Die Abbrandoberflaeche darf groesser aber nicht kleiner
		# als notwendig sein, maximaler Ueberschuss 0.5 m^2
		if dA >= delta_A && abs(dA - delta_A) < 0.5
			x(1) = A_tat + dA;	# Gesamtoberflaeche
			x(2) = l_zyl + l_end + dl + n * 0.03; # Gesamtlaenge
			x(3) = n;		# Anzahl der Spalte
			x(4) = alpha;		# Winkel
			x(5) = l_zyl + dl;	# Laenge des zyl. Teils
			x(6) = l_zyl + dl + n * 0.03;	# Gesamtlaenge des zyl. Teils
			# add to list
			spaltgeom = [spaltgeom; x];
		endif
	endfor
endfor
# strip the first empty value
spaltgeom = spaltgeom(2:end,:);
# find the best
[x k]=min(abs(spaltgeom(:,1) - delta_A));
# print
spaltgeom(k,:)
# Spaltwinkel
alpha = spaltgeom(k, 4);
# Anzahl der Spalte
n = spaltgeom(k, 3);
# Effektive Laenge des zylindrischen Teils
l_zyl_eff = spaltgeom(k, 5);
# Gesamtlaenge des zylindrischen Teils
l_zyl_gesamt = spaltgeom(k, 6);

# laengenregression: dl = n * dot_r * t * cos(alpha) + dl_endstuck
# dickenregression: db = dot_r * t;

abbrandprofil = zeros(1, 1);	# init
time = 0:0.1:t_b;
for t = time
	A = regression(t, r_dot, D_a/2, D_i/2, n, d2r(alpha), l_zyl_eff);
	abbrandprofil = [abbrandprofil; A];
endfor
abbrandprofil = abbrandprofil(2:end,:);	# strip the first row

figure(1);
plot(time, abbrandprofil, "0");
xlabel("Brennzeit [s]");
ylabel("Abbrandoberflaeche [m^2]");
grid on;
print abbrandprofil.eps;

schubprofil = r_dot * abbrandprofil * rho_TS * c_e;
figure(2);
plot(time, schubprofil/1000, "0");	# [kN]
xlabel("Brennzeit [s]");
ylabel("Schub [kN]");
grid on;
print schubprofil.eps;

[A_max k] = max(abbrandprofil)
t_max = time(k)
A_ende = abbrandprofil(end)
A_anfang = abbrandprofil(1)


########################################################################
#
# Aufgabe 3.2
#

Gamma = sqrt(kappa) * (2 / (kappa + 1))^((kappa + 1)/(2 * (kappa - 1)))
R = RR / M_TS
c_star = sqrt(R * T_c)/Gamma
A_t = c_star * m_dot / p_c
c_f = c_e / c_star

# Duseninnenduchmesser
D_t = 2 * sqrt(A_t / pi)

PI = p_c / p_e
A_e = A_t * (sqrt((kappa-1)/2) * (2 / (kappa+1))^((kappa+1)/(2*(kappa-1)))) / \
	(PI^(-1/kappa) * sqrt(1 - PI^((1-kappa)/kappa)))

# Dusenaussendurchmesser
D_e = 2 * sqrt(A_e / pi)

# Duesenlaenge
L_D = (0.9436 + 0.2524 * (A_e / A_t) \
	- 0.0022 * (A_e / A_t)^2 \
	- 6.06 * 10^(-7) * (A_e / A_t)^3) * D_t
# Klemmung
K_a = A_e / A_t

########################################################################
#
# Aufgabe 3.3
#

function [M V] = masse(ra, ri, rho, rt, l_zyl)
# [M V] = masse(ra, ri, rho, rt, l_zyl)
# rt - Duesenradius
# l_zyl - effektive Laenge des zylindrischen Teils
	V_kugel = 4/3 * pi * (ra^3 - ri^3);
	V_zylinder = pi * (ra^2 - ri^2) * l_zyl;
	# Naeherung der Oeffung fuer die Duese als Zylinder
	V_duesenausschnitt = pi * rt^2 * (ra - ri);
	V = V_kugel + V_zylinder - V_duesenausschnitt;
	M = V * rho;
endfunction

M_housing = masse(D_b/2, D_b/2 - d_w, rho_housing, D_t/2, l_zyl_gesamt)
M_iso = masse(D_b/2 - d_w, D_b/2 - d_w - d_iso, rho_iso, D_t/2, l_zyl_gesamt)
# Duesenmasse
M_d = 0.03 * sqrt(m/t_b) * L_D * 100	# L_D in [cm]

function V = freiesvolumen(V_ts, ra, l_zyl)
	V_kugel = 4/3 * pi * ra^3;
	V_zylinder = pi * ra^2 * l_zyl;
	V = V_kugel + V_zylinder - V_ts;
endfunction

V_ign = freiesvolumen(V, D_a/2, l_zyl_gesamt)
M_ign = 0.5 * (V_ign / 0.0254^3)^0.7 / 1000	# V_ign in [in^3] -> [kg]
M_ges = m + M_housing + M_iso + M_d + 0.10 * M_housing
gamma = m / M_ges	# Treibstoffmassenverhaeltnis
schubzugewicht = F / (M_ges * g)

# EOF
